@@ -19,22 +19,25 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 
 using System;
+using System.ComponentModel;
 using System.Data;
 using System.Data.Common;
-using System.Collections.Specialized;
+#if !CF
+using System.Drawing;
+using System.Drawing.Design;
+using System.Transactions;
+#endif
 using System.Text;
-using System.ComponentModel;
-using System.Globalization;
+using IsolationLevel=System.Data.IsolationLevel;
 using MySql.Data.Common;
 using System.Diagnostics;
-using System.Collections.Generic;
 
 namespace MySql.Data.MySqlClient
 {
     /// <include file='docs/MySqlConnection.xml' path='docs/ClassSummary/*'/>
-#if !PocketPC
-    [System.Drawing.ToolboxBitmap(typeof(MySqlConnection), "MySqlClient.resources.connection.bmp")]
-    [System.ComponentModel.DesignerCategory("Code")]
+#if !CF
+    [ToolboxBitmap(typeof (MySqlConnection), "MySqlClient.resources.connection.bmp")]
+    [DesignerCategory("Code")]
     [ToolboxItem(true)]
 #endif
     public sealed class MySqlConnection : DbConnection, ICloneable
@@ -47,20 +50,17 @@ namespace MySql.Data.MySqlClient
         private bool hasBeenOpen;
         private SchemaProvider schemaProvider;
         private ProcedureCache procedureCache;
+#if !CF
         private PerformanceMonitor perfMonitor;
-        private MySqlPromotableTransaction currentTransaction;
+#endif
         private bool isExecutingBuggyQuery;
         private string database;
 
         /// <include file='docs/MySqlConnection.xml' path='docs/InfoMessage/*'/>
         public event MySqlInfoMessageEventHandler InfoMessage;
 
-        private static Cache connectionStringCache = new Cache(0, 25);
-
-#if MONO2
-        /// <include file='docs/MySqlConnection.xml' path='docs/StateChange/*'/>
-        public event StateChangeEventHandler StateChange;
-#endif
+        private static Cache<string, MySqlConnectionStringBuilder> connectionStringCache =
+            new Cache<string, MySqlConnectionStringBuilder>(0, 25);
 
         /// <include file='docs/MySqlConnection.xml' path='docs/DefaultCtor/*'/>
         public MySqlConnection()
@@ -80,12 +80,14 @@ namespace MySql.Data.MySqlClient
 
         #region Interal Methods & Properties
 
-        internal MySqlPromotableTransaction CurrentTransaction
+#if !CF
+        internal PerformanceMonitor PerfMonitor
         {
-            get { return currentTransaction; }
-            set { currentTransaction = value; }
+            get { return perfMonitor; }
         }
 
+#endif
+
         internal ProcedureCache ProcedureCache
         {
             get { return procedureCache; }
@@ -102,11 +104,6 @@ namespace MySql.Data.MySqlClient
             set { dataReader = value; }
         }
 
-        internal char ParameterMarker
-        {
-            get { if (settings.UseOldSyntax) return '@'; return '?'; }
-        }
-
         internal void OnInfoMessage(MySqlInfoMessageEventArgs args)
         {
             if (InfoMessage != null)
@@ -115,25 +112,34 @@ namespace MySql.Data.MySqlClient
             }
         }
 
-        internal PerformanceMonitor PerfMonitor
-        {
-            get { return perfMonitor; }
-        }
-
         internal bool IsExecutingBuggyQuery
         {
             get { return isExecutingBuggyQuery; }
             set { isExecutingBuggyQuery = value; }
         }
 
+        internal bool SoftClosed
+        {
+            get 
+            {
+#if !CF
+                return (State == ConnectionState.Closed) && 
+                    driver != null && 
+                    driver.CurrentTransaction != null;
+#else
+                return false;            
+#endif
+            }
+        }
+
         #endregion
 
         #region Properties
 
-#if !PocketPC
+#if !CF
         [Browsable(false)]
 #endif
-        internal UsageAdvisor UsageAdvisor
+            internal UsageAdvisor UsageAdvisor
         {
             get { return advisor; }
         }
@@ -141,10 +147,10 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Returns the id of the server thread this connection is executing on
         /// </summary>
-#if !PocketPC
+#if !CF
         [Browsable(false)]
 #endif
-        public int ServerThread
+            public int ServerThread
         {
             get { return driver.ThreadID; }
         }
@@ -152,28 +158,28 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets the name of the MySQL server to which to connect.
         /// </summary>
-#if !PocketPC
+#if !CF
         [Browsable(true)]
 #endif
-        public override string DataSource
+            public override string DataSource
         {
             get { return settings.Server; }
         }
 
         /// <include file='docs/MySqlConnection.xml' path='docs/ConnectionTimeout/*'/>
-#if !PocketPC
+#if !CF
         [Browsable(true)]
 #endif
-        public override int ConnectionTimeout
+            public override int ConnectionTimeout
         {
-            get { return (int)settings.ConnectionTimeout; }
+            get { return (int) settings.ConnectionTimeout; }
         }
 
         /// <include file='docs/MySqlConnection.xml' path='docs/Database/*'/>
-#if !PocketPC
+#if !CF
         [Browsable(true)]
 #endif
-        public override string Database
+            public override string Database
         {
             get { return database; }
         }
@@ -181,28 +187,28 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Indicates if this connection should use compression when communicating with the server.
         /// </summary>
-#if !PocketPC
+#if !CF
         [Browsable(false)]
 #endif
-        public bool UseCompression
+            public bool UseCompression
         {
             get { return settings.UseCompression; }
         }
 
         /// <include file='docs/MySqlConnection.xml' path='docs/State/*'/>
-#if !PocketPC
+#if !CF
         [Browsable(false)]
 #endif
-        public override ConnectionState State
+            public override ConnectionState State
         {
             get { return connectionState; }
         }
 
         /// <include file='docs/MySqlConnection.xml' path='docs/ServerVersion/*'/>
-#if !PocketPC
+#if !CF
         [Browsable(false)]
 #endif
-        public override string ServerVersion
+            public override string ServerVersion
         {
             get { return driver.Version.ToString(); }
         }
@@ -212,7 +218,7 @@ namespace MySql.Data.MySqlClient
             get
             {
                 if (driver == null)
-                    return System.Text.Encoding.Default;
+                    return Encoding.Default;
                 else
                     return driver.Encoding;
             }
@@ -220,13 +226,14 @@ namespace MySql.Data.MySqlClient
 
 
         /// <include file='docs/MySqlConnection.xml' path='docs/ConnectionString/*'/>
-#if !PocketPC
-        [Editor("MySql.Data.MySqlClient.Design.ConnectionStringTypeEditor,MySqlClient.Design", typeof(System.Drawing.Design.UITypeEditor))]
+#if !CF
+        [Editor("MySql.Data.MySqlClient.Design.ConnectionStringTypeEditor,MySqlClient.Design", typeof (UITypeEditor))]
         [Browsable(true)]
         [Category("Data")]
-        [Description("Information used to connect to a DataSource, such as 'Server=xxx;UserId=yyy;Password=zzz;Database=dbdb'.")]
+        [Description(
+            "Information used to connect to a DataSource, such as 'Server=xxx;UserId=yyy;Password=zzz;Database=dbdb'.")]
 #endif
-        public override string ConnectionString
+            public override string ConnectionString
         {
             get
             {
@@ -236,7 +243,7 @@ namespace MySql.Data.MySqlClient
             }
             set
             {
-                if (this.State != ConnectionState.Closed)
+                if (State != ConnectionState.Closed)
                     throw new MySqlException(
                         "Not allowed to change the 'ConnectionString' property while the connection (state=" + State +
                         ").");
@@ -266,30 +273,61 @@ namespace MySql.Data.MySqlClient
 
         #region Transactions
 
-#if !MONO
+#if !MONO && !CF
         /// <summary>
         /// Enlists in the specified transaction. 
         /// </summary>
         /// <param name="transaction">
         /// A reference to an existing <see cref="System.Transactions.Transaction"/> in which to enlist.
         /// </param>
-        public override void EnlistTransaction(System.Transactions.Transaction transaction)
+        public override void EnlistTransaction(Transaction transaction)
         {
-            // if the transaction given to us is null, then there is nothing to do.
+            // enlisting in the null transaction is a noop
             if (transaction == null)
                 return;
 
-            if (currentTransaction != null)
+            // guard against trying to enlist in more than one transaction
+            if (driver.CurrentTransaction != null)
             {
-                if (currentTransaction.BaseTransaction == transaction)
+                if (driver.CurrentTransaction.BaseTransaction == transaction)
                     return;
 
                 throw new MySqlException("Already enlisted");
             }
 
-            MySqlPromotableTransaction t = new MySqlPromotableTransaction(this, transaction);
-            transaction.EnlistPromotableSinglePhase(t);
-            currentTransaction = t;
+            // now see if we need to swap out drivers.  We would need to do this since
+            // we have to make sure all ops for a given transaction are done on the
+            // same physical connection.
+            Driver existingDriver = DriverTransactionManager.GetDriverInTransaction(transaction);
+            if (existingDriver != null)
+            {
+                // we can't allow more than one driver to contribute to the same connection
+                if (existingDriver.IsInActiveUse)
+                    throw new NotSupportedException(Resources.MultipleConnectionsInTransactionNotSupported);
+
+                // there is an existing driver and it's not being currently used.
+                // now we need to see if it is using the same connection string
+                string text1 = existingDriver.Settings.GetConnectionString(true);
+                string text2 = Settings.GetConnectionString(true);
+                if (String.Compare(text1, text2, true) != 0)
+                    throw new NotSupportedException(Resources.MultipleConnectionsInTransactionNotSupported);
+
+                // close existing driver
+                // set this new driver as our existing driver
+                CloseFully();
+                driver = existingDriver;
+            }
+
+            if (driver.CurrentTransaction == null)
+            {
+                MySqlPromotableTransaction t = new MySqlPromotableTransaction(this, transaction);
+                if (!transaction.EnlistPromotableSinglePhase(t))
+                    throw new NotSupportedException(Resources.DistributedTxnNotSupported);
+
+                driver.CurrentTransaction = t;
+                DriverTransactionManager.SetDriverInTransaction(driver);
+                driver.IsInActiveUse = true;
+            }
         }
 #endif
 
@@ -318,13 +356,17 @@ namespace MySql.Data.MySqlClient
             switch (iso)
             {
                 case IsolationLevel.ReadCommitted:
-                    cmd.CommandText += "READ COMMITTED"; break;
+                    cmd.CommandText += "READ COMMITTED";
+                    break;
                 case IsolationLevel.ReadUncommitted:
-                    cmd.CommandText += "READ UNCOMMITTED"; break;
+                    cmd.CommandText += "READ UNCOMMITTED";
+                    break;
                 case IsolationLevel.RepeatableRead:
-                    cmd.CommandText += "REPEATABLE READ"; break;
+                    cmd.CommandText += "REPEATABLE READ";
+                    break;
                 case IsolationLevel.Serializable:
-                    cmd.CommandText += "SERIALIZABLE"; break;
+                    cmd.CommandText += "SERIALIZABLE";
+                    break;
                 case IsolationLevel.Chaos:
                     throw new NotSupportedException(Resources.ChaosNotSupported);
             }
@@ -340,25 +382,26 @@ namespace MySql.Data.MySqlClient
         #endregion
 
         /// <include file='docs/MySqlConnection.xml' path='docs/ChangeDatabase/*'/>
-        public override void ChangeDatabase(string database)
+        public override void ChangeDatabase(string databaseName)
         {
-            if (database == null || database.Trim().Length == 0)
-                throw new ArgumentException(Resources.ParameterIsInvalid, "database");
+            if (databaseName == null || databaseName.Trim().Length == 0)
+                throw new ArgumentException(Resources.ParameterIsInvalid, "databaseName");
 
             if (State != ConnectionState.Open)
                 throw new InvalidOperationException(Resources.ConnectionNotOpen);
 
-            driver.SetDatabase(database);
-            this.database = database;
+            driver.SetDatabase(databaseName);
+            this.database = databaseName;
         }
 
-        internal void SetState(ConnectionState newConnectionState)
+        internal void SetState(ConnectionState newConnectionState, bool broadcast)
         {
-            if (newConnectionState == this.connectionState)
+            if (newConnectionState == connectionState && !broadcast)
                 return;
-            ConnectionState oldConnectionState = this.connectionState;
-            this.connectionState = newConnectionState;
-            this.OnStateChange(new StateChangeEventArgs(oldConnectionState, this.connectionState));
+            ConnectionState oldConnectionState = connectionState;
+            connectionState = newConnectionState;
+			if (broadcast)
+				OnStateChange(new StateChangeEventArgs(oldConnectionState, connectionState));
         }
 
         /// <summary>
@@ -367,10 +410,11 @@ namespace MySql.Data.MySqlClient
         /// <returns></returns>
         public bool Ping()
         {
-            bool result = driver.Ping();
-            if (!result)
-                SetState(ConnectionState.Closed);
-            return result;
+            if (driver != null && driver.Ping())
+                return true;
+            driver = null;
+            SetState(ConnectionState.Closed, true);
+            return false;
         }
 
         /// <include file='docs/MySqlConnection.xml' path='docs/Open/*'/>
@@ -379,25 +423,40 @@ namespace MySql.Data.MySqlClient
             if (State == ConnectionState.Open)
                 throw new InvalidOperationException(Resources.ConnectionAlreadyOpen);
 
-            SetState(ConnectionState.Connecting);
+            SetState(ConnectionState.Connecting, true);
+
+#if !CF
+                // if we are auto enlisting in a current transaction, then we will be
+                // treating the connection as pooled
+                if (settings.AutoEnlist && Transaction.Current != null)
+                {
+                    driver = DriverTransactionManager.GetDriverInTransaction(Transaction.Current);
+                    if (driver != null &&
+                        (driver.IsInActiveUse ||
+                        !driver.Settings.EquivalentTo(this.Settings)))
+                        throw new NotSupportedException(Resources.MultipleConnectionsInTransactionNotSupported);
+                }
+#endif
 
             try
             {
                 if (settings.Pooling)
                 {
                     MySqlPool pool = MySqlPoolManager.GetPool(settings);
-                    driver = pool.GetConnection();
+                    if (driver == null)
+                        driver = pool.GetConnection();
                     procedureCache = pool.ProcedureCache;
                 }
                 else
                 {
-                    driver = Driver.Create(settings);
-                    procedureCache = new ProcedureCache(settings.ProcedureCacheSize);
+                    if (driver == null)
+                        driver = Driver.Create(settings);
+                    procedureCache = new ProcedureCache((int) settings.ProcedureCacheSize);
                 }
             }
             catch (Exception)
             {
-                SetState(ConnectionState.Closed);
+                SetState(ConnectionState.Closed, true);
                 throw;
             }
 
@@ -405,7 +464,7 @@ namespace MySql.Data.MySqlClient
             if (driver.Settings.UseOldSyntax)
                 Logger.LogWarning("You are using old syntax that will be removed in future versions");
 
-            SetState(ConnectionState.Open);
+            SetState(ConnectionState.Open, false);
             driver.Configure(this);
             if (settings.Database != null && settings.Database != String.Empty)
                 ChangeDatabase(settings.Database);
@@ -415,17 +474,20 @@ namespace MySql.Data.MySqlClient
                 schemaProvider = new ISSchemaProvider(this);
             else
                 schemaProvider = new SchemaProvider(this);
+#if !CF
             perfMonitor = new PerformanceMonitor(this);
+#endif
 
             // if we are opening up inside a current transaction, then autoenlist
             // TODO: control this with a connection string option
-#if !MONO
-            if (System.Transactions.Transaction.Current != null)
-                EnlistTransaction(System.Transactions.Transaction.Current);
+#if !MONO && !CF
+            if (Transaction.Current != null && settings.AutoEnlist)
+                EnlistTransaction(Transaction.Current);
 #endif
 
             hasBeenOpen = true;
-        }
+			SetState(ConnectionState.Open, true);
+		}
 
         /// <include file='docs/MySqlConnection.xml' path='docs/CreateCommand/*'/>
         public new MySqlCommand CreateCommand()
@@ -437,6 +499,7 @@ namespace MySql.Data.MySqlClient
         }
 
         #region ICloneable
+
         /// <summary>
         /// Creates a new MySqlConnection object with the exact same ConnectionString value
         /// </summary>
@@ -444,9 +507,12 @@ namespace MySql.Data.MySqlClient
         object ICloneable.Clone()
         {
             MySqlConnection clone = new MySqlConnection();
-            clone.ConnectionString = settings.GetConnectionString(true);
+            string connectionString = settings.GetConnectionString(true);
+            if (connectionString != null)
+                clone.ConnectionString = connectionString;
             return clone;
         }
+
         #endregion
 
         #region IDisposeable
@@ -481,18 +547,14 @@ namespace MySql.Data.MySqlClient
                 else
                     driver.Close();
             }
-            catch (Exception) { }
-            SetState(ConnectionState.Closed);
+            catch (Exception)
+            {
+            }
+            SetState(ConnectionState.Closed, true);
         }
 
-        /// <include file='docs/MySqlConnection.xml' path='docs/Close/*'/>
-        public override void Close()
+        internal void CloseFully()
         {
-            if (State == ConnectionState.Closed) return;
-
-            if (dataReader != null)
-                dataReader.Close();
-
             if (settings.Pooling)
             {
                 // if we are in a transaction, roll it back
@@ -506,19 +568,41 @@ namespace MySql.Data.MySqlClient
             }
             else
                 driver.Close();
-
-            SetState(ConnectionState.Closed);
+            driver = null;
         }
 
-#if MONO2
-
-        protected void OnStateChange (StateChangeEventArgs stateChangeArgs)
+        /// <include file='docs/MySqlConnection.xml' path='docs/Close/*'/>
+        public override void Close()
         {
-            if (StateChange != null)
-                StateChange(this, stateChangeArgs);
+            if (State == ConnectionState.Closed) return;
+
+            if (dataReader != null)
+                dataReader.Close();
+
+			// if the reader was opened with CloseConnection then driver
+			// will be null on the second time through
+			if (driver != null)
+			{
+#if !CF
+				if (driver.CurrentTransaction == null)
+#endif	
+			        CloseFully();
+#if !CF
+				else
+					driver.IsInActiveUse = false;
+#endif
+			}
+
+            SetState(ConnectionState.Closed, true);
         }
 
-#endif
+		internal string CurrentDatabase()
+		{
+			if (Database != null && Database.Length > 0)
+				return Database;
+			MySqlCommand cmd = new MySqlCommand("SELECT database()", this);
+			return cmd.ExecuteScalar().ToString();
+		}
 
         #region GetSchema Support
 
@@ -555,23 +639,25 @@ namespace MySql.Data.MySqlClient
         /// <returns>A <see cref="DataTable"/> that contains schema information.</returns>
         public override DataTable GetSchema(string collectionName, string[] restrictionValues)
         {
-            /*            string msg = String.Format("collection = {0}", collectionName);
-                        foreach (string s in restrictionValues)
-                        {
-                            msg += String.Format(" res={0}", s);
-                        }
-                        MessageBox.Show(msg);
-              */
+/*            string msg = String.Format("collection name2 = {0}", collectionName);
+            if (restrictionValues != null)
+                foreach (string s in restrictionValues)
+                {
+                    msg += String.Format(" res={0}", s);
+                }
+            System.Windows.Forms.MessageBox.Show(msg);
+            */
             if (collectionName == null)
                 collectionName = SchemaProvider.MetaCollection;
 
             string[] restrictions = null;
             if (restrictionValues != null)
             {
-                restrictions = new string[restrictionValues.Length];
-                for (int x = 0; x < restrictionValues.Length; x++)
+                restrictions = (string[]) restrictionValues.Clone();
+
+                for (int x = 0; x < restrictions.Length; x++)
                 {
-                    string s = restrictionValues[x];
+                    string s = restrictions[x];
                     if (s != null)
                     {
                         if (s.StartsWith("`"))
@@ -589,6 +675,21 @@ namespace MySql.Data.MySqlClient
 
         #endregion
 
+        #region Pool Routines
+
+        /// <include file='docs/MySqlConnection.xml' path='docs/ClearPool/*'/>
+        public static void ClearPool(MySqlConnection connection)
+        {
+            MySqlPoolManager.ClearPool(connection.Settings);
+        }
+
+        /// <include file='docs/MySqlConnection.xml' path='docs/ClearAllPools/*'/>
+        public static void ClearAllPools()
+        {
+            MySqlPoolManager.ClearAllPools();
+        }
+
+        #endregion
     }
 
     /// <summary>
