@@ -21,17 +21,22 @@
 using System;
 using System.Data;
 using System.Data.Common;
+using System.IO;
 using System.Collections;
+using System.Text;
+using MySql.Data.Common;
 using System.ComponentModel;
 using System.Threading;
+using System.Diagnostics;
 using System.Globalization;
+using System.Collections.Generic;
 
 namespace MySql.Data.MySqlClient
 {
 	/// <include file='docs/mysqlcommand.xml' path='docs/ClassSummary/*'/>
-#if !PocketPC
+#if !CF
 	[System.Drawing.ToolboxBitmap(typeof(MySqlCommand), "MySqlClient.resources.command.bmp")]
-	[DesignerCategory("Code")]
+	[System.ComponentModel.DesignerCategory("Code")]
 #endif
 	public sealed class MySqlCommand : DbCommand, ICloneable
 	{
@@ -51,17 +56,18 @@ namespace MySql.Data.MySqlClient
 		private bool canCancel;
 		private bool timedOut, canceled;
         private bool resetSqlSelect;
+        List<MySqlCommand> batch;
+        private string batchableCommandText;
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/ctor1/*'/>
 		public MySqlCommand()
 		{
 			designTimeVisible = true;
 			cmdType = CommandType.Text;
-			parameters = new MySqlParameterCollection();
+			parameters = new MySqlParameterCollection(this);
 			updatedRowSource = UpdateRowSource.Both;
 			cursorPageSize = 0;
 			cmdText = String.Empty;
-			commandTimeout = 30;
 			canCancel = false;
 			timedOut = false;
 		}
@@ -78,8 +84,6 @@ namespace MySql.Data.MySqlClient
 			: this(cmdText)
 		{
 			Connection = connection;
-			if (connection != null)
-				parameters.ParameterMarker = connection.ParameterMarker;
 		}
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/ctor4/*'/>
@@ -95,7 +99,7 @@ namespace MySql.Data.MySqlClient
 
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/LastInseredId/*'/>
-#if !PocketPC
+#if !CF
 		[Browsable(false)]
 #endif
 		public Int64 LastInsertedId
@@ -104,7 +108,7 @@ namespace MySql.Data.MySqlClient
 		}
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/CommandText/*'/>
-#if !PocketPC
+#if !CF
 		[Category("Data")]
 		[Description("Command text to execute")]
 		[Editor("MySql.Data.Common.Design.SqlCommandTextEditor,MySqlClient.Design", typeof(System.Drawing.Design.UITypeEditor))]
@@ -116,7 +120,7 @@ namespace MySql.Data.MySqlClient
 			{
 				cmdText = value;
 				statement = null;
-				if (cmdText.EndsWith("DEFAULT VALUES"))
+				if (cmdText != null && cmdText.EndsWith("DEFAULT VALUES"))
 				{
 					cmdText = cmdText.Substring(0, cmdText.Length - 14);
 					cmdText = cmdText + "() VALUES ()";
@@ -131,19 +135,19 @@ namespace MySql.Data.MySqlClient
 		}
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/CommandTimeout/*'/>
-#if !PocketPC
+#if !CF
 		[Category("Misc")]
 		[Description("Time to wait for command to execute")]
 		[DefaultValue(30)]
 #endif
 		public override int CommandTimeout
 		{
-			get { return commandTimeout; }
+			get { return commandTimeout == 0 ? 30 : commandTimeout; }
 			set { commandTimeout = value; }
 		}
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/CommandType/*'/>
-#if !PocketPC
+#if !CF
 		[Category("Data")]
 #endif
 		public override CommandType CommandType
@@ -153,7 +157,7 @@ namespace MySql.Data.MySqlClient
 		}
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/IsPrepared/*'/>
-#if !PocketPC
+#if !CF
 		[Browsable(false)]
 #endif
 		public bool IsPrepared
@@ -162,7 +166,7 @@ namespace MySql.Data.MySqlClient
 		}
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/Connection/*'/>
-#if !PocketPC
+#if !CF
 		[Category("Behavior")]
 		[Description("Connection used by the command")]
 #endif
@@ -177,16 +181,19 @@ namespace MySql.Data.MySqlClient
 				* is reset.
 				*/
 				if (connection != value)
-					Transaction = null;
+					this.Transaction = null;
 
 				connection = (MySqlConnection)value;
-				if (connection != null)
-					parameters.ParameterMarker = connection.ParameterMarker;
+
+                // if the user has not already set the command timeout, then
+                // take the default from the connection
+                if (connection != null && commandTimeout == 0)
+                    commandTimeout = (int)connection.Settings.DefaultCommandTimeout;
 			}
 		}
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/Parameters/*'/>
-#if !PocketPC
+#if !CF
 		[Category("Data")]
 		[Description("The parameters collection")]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
@@ -198,17 +205,17 @@ namespace MySql.Data.MySqlClient
 
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/Transaction/*'/>
-#if !PocketPC
+#if !CF
 		[Browsable(false)]
 #endif
 		public new MySqlTransaction Transaction
 		{
 			get { return curTransaction; }
-			set { curTransaction = value; }
+			set { curTransaction = (MySqlTransaction)value; }
 		}
 
 		/*		/// <include file='docs/mysqlcommand.xml' path='docs/UpdatedRowSource/*'/>
-		#if !PocketPC
+		#if !CF
 				[Category("Behavior")]
 		#endif
 				public override UpdateRowSource UpdatedRowSource
@@ -216,6 +223,17 @@ namespace MySql.Data.MySqlClient
 					get { return updatedRowSource;  }
 					set { updatedRowSource = value; }
 				}*/
+
+        internal List<MySqlCommand> Batch
+        {
+            get { return batch; }
+        }
+
+        internal string BatchableCommandText
+        {
+            get { return batchableCommandText; }
+        }
+
 		#endregion
 
 		#region Methods
@@ -267,7 +285,8 @@ namespace MySql.Data.MySqlClient
 		private void CheckState()
 		{
 			// There must be a valid and open connection.
-			if (connection == null || connection.State != ConnectionState.Open)
+			if ((connection == null || connection.State != ConnectionState.Open) && 
+                !connection.SoftClosed)
 				throw new InvalidOperationException("Connection must be valid and open");
 
 			// Data readers have to be closed first
@@ -289,7 +308,7 @@ namespace MySql.Data.MySqlClient
 			{
 				reader.Close();
 				lastInsertedId = reader.InsertedId;
-				updatedRowCount = reader.RecordsAffected;
+				this.updatedRowCount = reader.RecordsAffected;
 			}
 			return (int)updatedRowCount;
 		}
@@ -477,7 +496,7 @@ namespace MySql.Data.MySqlClient
         }
 
 		/// <include file='docs/mysqlcommand.xml' path='docs/Prepare2/*'/>
-		private void Prepare(int pageSize)
+		private void Prepare(int cursorPageSize)
 		{
 			if (!connection.driver.Version.isAtLeast(5, 0, 0) && cursorPageSize > 0)
 				throw new InvalidOperationException("Nested commands are only supported on MySQL 5.0 and later");
@@ -513,8 +532,8 @@ namespace MySql.Data.MySqlClient
 
 		#region Async Methods
 
-		internal delegate int AsyncExecuteNonQueryDelegate();
-		internal delegate MySqlDataReader AsyncExecuteReaderDelegate(CommandBehavior behavior);
+		internal delegate void AsyncDelegate(int type, CommandBehavior behavior);
+		internal Exception thrownException;
 
 		private static string TrimSemicolons(string sql)
 		{
@@ -529,6 +548,22 @@ namespace MySql.Data.MySqlClient
 			return sb.ToString(start, end - start + 1);
 		}
 
+		internal void AsyncExecuteWrapper(int type, CommandBehavior behavior)
+		{
+			thrownException = null;
+			try
+			{
+				if (type == 1)
+					ExecuteReader(behavior);
+				else
+					ExecuteNonQuery();
+			}
+			catch (Exception ex)
+			{
+				thrownException = ex;
+			}
+		}
+
 		/// <summary>
 		/// Initiates the asynchronous execution of the SQL statement or stored procedure 
 		/// that is described by this <see cref="MySqlCommand"/>, and retrieves one or more 
@@ -556,8 +591,8 @@ namespace MySql.Data.MySqlClient
 		/// the returned rows. </returns>
 		public IAsyncResult BeginExecuteReader(CommandBehavior behavior)
 		{
-			AsyncExecuteReaderDelegate del = new AsyncExecuteReaderDelegate(ExecuteReader);
-			asyncResult = del.BeginInvoke(behavior, null, null);
+			AsyncDelegate del = new AsyncDelegate(AsyncExecuteWrapper);
+			asyncResult = del.BeginInvoke(1, behavior, null, null);
 			return asyncResult;
 		}
 
@@ -571,6 +606,8 @@ namespace MySql.Data.MySqlClient
 		public MySqlDataReader EndExecuteReader(IAsyncResult result)
 		{
 			result.AsyncWaitHandle.WaitOne();
+			if (thrownException != null)
+				throw thrownException;
 			return connection.Reader;
 		}
 
@@ -590,9 +627,9 @@ namespace MySql.Data.MySqlClient
 		/// which returns the number of affected rows. </returns>
 		public IAsyncResult BeginExecuteNonQuery(AsyncCallback callback, object stateObject)
 		{
-			AsyncExecuteNonQueryDelegate del =
-				 new AsyncExecuteNonQueryDelegate(ExecuteNonQuery);
-			asyncResult = del.BeginInvoke(callback, stateObject);
+			AsyncDelegate del = new AsyncDelegate(AsyncExecuteWrapper);
+			asyncResult = del.BeginInvoke(2, CommandBehavior.Default, 
+				callback, stateObject);
 			return asyncResult;
 		}
 
@@ -605,21 +642,22 @@ namespace MySql.Data.MySqlClient
 		/// which returns the number of affected rows. </returns>
 		public IAsyncResult BeginExecuteNonQuery()
 		{
-			AsyncExecuteNonQueryDelegate del =
-				new AsyncExecuteNonQueryDelegate(ExecuteNonQuery);
-			asyncResult = del.BeginInvoke(null, null);
+			AsyncDelegate del = new AsyncDelegate(AsyncExecuteWrapper);
+			asyncResult = del.BeginInvoke(2, CommandBehavior.Default, null, null);
 			return asyncResult;
 		}
 
 		/// <summary>
 		/// Finishes asynchronous execution of a SQL statement. 
 		/// </summary>
-		/// <param name="asResult">The <see cref="IAsyncResult"/> returned by the call 
+		/// <param name="asyncResult">The <see cref="IAsyncResult"/> returned by the call 
 		/// to <see cref="BeginExecuteNonQuery()"/>.</param>
 		/// <returns></returns>
-		public int EndExecuteNonQuery(IAsyncResult asResult)
+		public int EndExecuteNonQuery(IAsyncResult asyncResult)
 		{
-			asResult.AsyncWaitHandle.WaitOne();
+			asyncResult.AsyncWaitHandle.WaitOne();
+			if (thrownException != null)
+				throw thrownException;
 			return (int)updatedRowCount;
 		}
 
@@ -676,6 +714,13 @@ namespace MySql.Data.MySqlClient
 					return buffers;
 				}*/
 
+        internal long EstimatedSize()
+        {
+            long size = CommandText.Length;
+            foreach (MySqlParameter parameter in Parameters)
+                size += parameter.EstimatedSize();
+            return size;
+        }
 
 		#endregion
 
@@ -690,6 +735,7 @@ namespace MySql.Data.MySqlClient
 			MySqlCommand clone = new MySqlCommand(cmdText, connection, curTransaction);
             clone.CommandType = CommandType;
             clone.CommandTimeout = CommandTimeout;
+            clone.batchableCommandText = batchableCommandText;
 
 			foreach (MySqlParameter p in parameters)
 			{
@@ -699,6 +745,63 @@ namespace MySql.Data.MySqlClient
 		}
 		#endregion
 
+        #region Batching support
+
+        internal void AddToBatch(MySqlCommand command)
+        {
+            if (batch == null)
+                batch = new List<MySqlCommand>();
+            batch.Add(command);
+        }
+
+        internal string GetCommandTextForBatching()
+        {
+            if (batchableCommandText == null)
+            {
+                // if the command starts with insert and is "simple" enough, then
+                // we can use the multi-value form of insert
+                if (String.Compare(CommandText.Substring(0, 6), "INSERT", true) == 0)
+                {
+                    MySqlCommand cmd = new MySqlCommand("SELECT @@sql_mode", Connection);
+                    string sql_mode = cmd.ExecuteScalar().ToString().ToLower(CultureInfo.InvariantCulture);
+                    SqlTokenizer tokenizer = new SqlTokenizer(CommandText);
+                    tokenizer.AnsiQuotes = sql_mode.IndexOf("ansi_quotes") != -1;
+                    tokenizer.BackslashEscapes = sql_mode.IndexOf("no_backslash_escapes") == -1;
+                    string token = tokenizer.NextToken().ToLower(CultureInfo.InvariantCulture);
+                    while (token != null)
+                    {
+                        if (token.ToLower(CultureInfo.InvariantCulture) == "values" && 
+                            !tokenizer.Quoted)
+                        {
+                            token = tokenizer.NextToken();
+                            Debug.Assert(token == "(");
+                            while (token != null && token != ")")
+                            {
+                                batchableCommandText += token;
+                                token = tokenizer.NextToken();
+                            }
+                            if (token != null)
+                                batchableCommandText += token;
+                            token = tokenizer.NextToken();
+                            if (token != null && (token == "," || 
+                                token.ToLower(CultureInfo.InvariantCulture) == "on"))
+                            {
+                                batchableCommandText = null;
+                                break;
+                            }
+                        }
+                        token = tokenizer.NextToken();
+                    }
+                }
+                if (batchableCommandText == null)
+                    batchableCommandText = CommandText;
+            }
+
+            return batchableCommandText;
+        }
+
+        #endregion
+
         protected override void Dispose(bool disposing)
         {
             if (disposing)
@@ -712,18 +815,18 @@ namespace MySql.Data.MySqlClient
         /// <summary>
         /// Gets or sets a value indicating whether the command object should be visible in a Windows Form Designer control. 
         /// </summary>
-#if !PocketPC
+#if !CF
         [Browsable(false)]
 #endif
 		public override bool DesignTimeVisible
 		{
 			get
 			{
-				return designTimeVisible;
+				return this.designTimeVisible;
 			}
 			set
 			{
-				designTimeVisible = value;
+				this.designTimeVisible = value;
 			}
 		}
 
@@ -735,11 +838,11 @@ namespace MySql.Data.MySqlClient
 		{
 			get
 			{
-				return updatedRowSource;
+				return this.updatedRowSource;
 			}
 			set
 			{
-				updatedRowSource = value;
+				this.updatedRowSource = value;
 			}
 		}
 
@@ -750,24 +853,24 @@ namespace MySql.Data.MySqlClient
 
 		protected override DbConnection DbConnection
 		{
-			get { return Connection; }
-			set { Connection = (MySqlConnection)value; }
+			get { return this.Connection; }
+			set { this.Connection = (MySqlConnection)value; }
 		}
 
 		protected override DbParameterCollection DbParameterCollection
 		{
-			get { return Parameters; }
+			get { return this.Parameters; }
 		}
 
 		protected override DbTransaction DbTransaction
 		{
-			get { return Transaction; }
-			set { Transaction = (MySqlTransaction)value; }
+			get { return this.Transaction; }
+			set { this.Transaction = (MySqlTransaction)value; }
 		}
 
 		protected override DbDataReader ExecuteDbDataReader(CommandBehavior behavior)
 		{
-			return ExecuteReader(behavior);
+			return this.ExecuteReader(behavior);
 		}
 	}
 }
