@@ -19,7 +19,6 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 
 using System;
-using System.Data;
 using System.Data.Common;
 using System.Collections;
 using System.ComponentModel;
@@ -27,10 +26,10 @@ using System.ComponentModel;
 namespace MySql.Data.MySqlClient
 {
 	/// <summary>
-	/// Represents a collection of parameters relevant to a <see cref="MySqlCommand"/> as well as their respective mappings to columns in a <see cref="DataSet"/>. This class cannot be inherited.
+	/// Represents a collection of parameters relevant to a <see cref="MySqlCommand"/> as well as their respective mappings to columns in a <see cref="System.Data.DataSet"/>. This class cannot be inherited.
 	/// </summary>
 	/// <include file='docs/MySqlParameterCollection.xml' path='MyDocs/MyMembers[@name="Class"]/*'/>
-#if !PocketPC
+#if !CF
 	[Editor("MySql.Data.MySqlClient.Design.DBParametersEditor,MySql.Design", typeof(System.Drawing.Design.UITypeEditor))]
 	[ListBindable(true)]
 #endif
@@ -38,9 +37,9 @@ namespace MySql.Data.MySqlClient
 	{
         private ArrayList items = new ArrayList();
         private Hashtable indexHash;
-		private char paramMarker = '?';
+        private MySqlCommand owningCommand;
 
-		internal MySqlParameterCollection()
+		internal MySqlParameterCollection(MySqlCommand cmd)
 		{
 #if NET20
 			indexHash = new Hashtable(StringComparer.CurrentCultureIgnoreCase);
@@ -49,12 +48,7 @@ namespace MySql.Data.MySqlClient
 			    new CaseInsensitiveComparer());
 #endif
             Clear();
-		}
-
-		internal char ParameterMarker
-		{
-			get { return paramMarker; }
-			set { paramMarker = value; }
+            owningCommand = cmd;
 		}
 
 		#region Public Methods
@@ -170,12 +164,13 @@ namespace MySql.Data.MySqlClient
 		/// <returns></returns>
 		protected override DbParameter GetParameter(string parameterName)
 		{
-			int index = this.IndexOf(parameterName);
+			int index = IndexOf(parameterName);
 			if (index < 0)
 			{
 				// check to see if the user has added the parameter without a
 				// parameter marker.  If so, kindly tell them what they did.
-				if (parameterName.StartsWith(ParameterMarker.ToString()))
+				if (parameterName.StartsWith("@") ||
+                    parameterName.StartsWith("?"))
 				{
 					string newParameterName = parameterName.Substring(1);
 					index = IndexOf(newParameterName);
@@ -195,7 +190,7 @@ namespace MySql.Data.MySqlClient
 
 		protected override void SetParameter(string parameterName, DbParameter value)
 		{
-			int index = this.IndexOf(parameterName);
+			int index = IndexOf(parameterName);
 			if (index < 0)
 				throw new ArgumentException("Parameter '" + parameterName + "' not found in the collection.");
             SetParameter(index, value);
@@ -207,7 +202,7 @@ namespace MySql.Data.MySqlClient
             MySqlParameter p = (MySqlParameter)items[index];
             
             indexHash.Remove(p.ParameterName);
-			items[index] = (MySqlParameter)value;
+			items[index] = value;
             indexHash.Add(value.ParameterName, index);
         }
 
@@ -244,11 +239,11 @@ namespace MySql.Data.MySqlClient
 		/// <summary>
 		/// Gets a value indicating whether a <see cref="MySqlParameter"/> with the specified parameter name exists in the collection.
 		/// </summary>
-		/// <param name="value">The name of the <see cref="MySqlParameter"/> object to find.</param>
+		/// <param name="parameterName">The name of the <see cref="MySqlParameter"/> object to find.</param>
 		/// <returns>true if the collection contains the parameter; otherwise, false.</returns>
-		public override bool Contains(string value)
+		public override bool Contains(string parameterName)
 		{
-			return IndexOf(value) != -1;
+			return IndexOf(parameterName) != -1;
 		}
 
 		/// <summary>
@@ -286,7 +281,7 @@ namespace MySql.Data.MySqlClient
         /// <returns></returns>
 		public override IEnumerator GetEnumerator()
 		{
-			return ((IEnumerable)items).GetEnumerator();
+			return items.GetEnumerator();
 		}
 
 		/// <summary>
@@ -369,10 +364,10 @@ namespace MySql.Data.MySqlClient
 		/// <summary>
 		/// Removes the specified <see cref="MySqlParameter"/> from the collection using the parameter name.
 		/// </summary>
-		/// <param name="name">The name of the <see cref="MySqlParameter"/> object to retrieve. </param>
-		public override void RemoveAt(string name)
+		/// <param name="parameterName">The name of the <see cref="MySqlParameter"/> object to retrieve. </param>
+		public override void RemoveAt(string parameterName)
 		{
-			DbParameter p = GetParameter(name);
+			DbParameter p = GetParameter(parameterName);
 			Remove(p);
 		}
 
@@ -415,7 +410,7 @@ namespace MySql.Data.MySqlClient
             if (indexHash.ContainsKey(inComingName))
                 throw new MySqlException(
                     String.Format(Resources.ParameterAlreadyDefined, value.ParameterName));
-            if (inComingName[0] == paramMarker)
+            if (inComingName[0] == '@' || inComingName[0] == '?')
                 inComingName = inComingName.Substring(1, inComingName.Length - 1);
             if (indexHash.ContainsKey(inComingName))
                 throw new MySqlException(
@@ -449,5 +444,23 @@ namespace MySql.Data.MySqlClient
                 indexHash[p.ParameterName] = addEntry ? ++index : --index;
             }
         }
+
+        internal MySqlParameter GetParameterFlexible(string parameterName, bool throwOnNotFound)
+        {
+            char firstChar = parameterName[0];
+            string baseName = parameterName;
+            if ('@' == firstChar || '?' == firstChar)
+                baseName = baseName.Substring(1);
+            int index = IndexOf("@" + baseName);
+            if (-1 == index)
+                index = IndexOf("?" + baseName);
+            if (-1 == index)
+                index = IndexOf(baseName);
+            if (-1 != index)
+                return this[index];
+            if (throwOnNotFound)
+                throw new ArgumentException("Parameter '" + parameterName + "' not found in the collection.");
+            return null;
+        }
 	}
 }
