@@ -22,7 +22,7 @@ using System;
 using MySql.Data.MySqlClient;
 using MySql.Data.Types;
 using System.Data;
-using NUnit.Framework;
+using MbUnit.Framework;
 
 namespace MySql.Data.MySqlClient.Tests
 {
@@ -33,7 +33,7 @@ namespace MySql.Data.MySqlClient.Tests
 	public class DataTypeTests : BaseTest
 	{
 		[SetUp]
-		protected override void Setup() 
+		public override void Setup() 
 		{
 			base.Setup();
 
@@ -45,13 +45,14 @@ namespace MySql.Data.MySqlClient.Tests
 		public void BytesAndBooleans() 
 		{
 			InternalBytesAndBooleans(false);
-		}
+        }
 
-		[Category("4.1")]
-		[Test]
-		public void BytesAndBooleansPrepared()
-		{
-			InternalBytesAndBooleans(true);
+        [Test]
+        public void BytesAndBooleansPrepared()
+        {
+            if (version < new Version(4, 1)) return;
+
+            InternalBytesAndBooleans(true);
 		}
 
 		private void InternalBytesAndBooleans(bool prepare) 
@@ -97,12 +98,13 @@ namespace MySql.Data.MySqlClient.Tests
 			InternalTestFloats(false);
 		}
 
-		[Test]
-		[Category("4.1")]
-		public void TestFloatPrepared()
-		{
-			InternalTestFloats(true);
-		}
+        [Test]
+        public void TestFloatPrepared()
+        {
+            if (version < new Version(4, 1)) return;
+
+            InternalTestFloats(true);
+        }
 
 		private void InternalTestFloats(bool prepared)
 		{
@@ -132,7 +134,7 @@ namespace MySql.Data.MySqlClient.Tests
 			{
 				cmd.CommandText = "SELECT * FROM Test";
 				if (prepared) 
-					cmd.Prepare();
+                    cmd.Prepare();
 				reader = cmd.ExecuteReader();
 				Assert.IsTrue(reader.Read());
 				Assert.AreEqual(2.3, (decimal)reader.GetFloat(0));
@@ -151,7 +153,7 @@ namespace MySql.Data.MySqlClient.Tests
 			finally 
 			{
 				if (reader != null) 
-					reader.Close();
+                    reader.Close();
 			}
 		}
 
@@ -268,7 +270,7 @@ namespace MySql.Data.MySqlClient.Tests
 			execSQL("INSERT INTO Test VALUES (98)");
 			execSQL("INSERT INTO Test VALUES (1990)");
 			execSQL("INSERT INTO Test VALUES (2004)");
-			execSQL("SET SQL_MODE=''");
+            execSQL("SET SQL_MODE=''");
 			execSQL("INSERT INTO Test VALUES (111111111111111111111)");
 
 			MySqlCommand cmd = new MySqlCommand("SELECT * FROM Test", conn);
@@ -292,7 +294,7 @@ namespace MySql.Data.MySqlClient.Tests
 			finally 
 			{
 				if (reader != null) 
-					reader.Close();
+                    reader.Close();
 			}
 		}
 
@@ -318,14 +320,14 @@ namespace MySql.Data.MySqlClient.Tests
 		[Test]
 		public void Timestamp() 
 		{
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (id int, dt DATETIME, ts2 TIMESTAMP(2), ts4 TIMESTAMP(4), " + 
+			execSQL("DROP TABLE IF EXISTS Test");
+			execSQL("CREATE TABLE Test (id int, dt DATETIME, ts2 TIMESTAMP(2), ts4 TIMESTAMP(4), " + 
 				"ts6 TIMESTAMP(6), ts8 TIMESTAMP(8), ts10 TIMESTAMP(10), ts12 TIMESTAMP(12), " +
 				"ts14 TIMESTAMP(14))");
-			execSQL("INSERT INTO test (id, dt, ts2, ts4, ts6, ts8, ts10, ts12, ts14) " +
+			execSQL("INSERT INTO Test (id, dt, ts2, ts4, ts6, ts8, ts10, ts12, ts14) " +
 				"VALUES (1, Now(), Now(), Now(), Now(), Now(), Now(), Now(), Now())");
 
-			MySqlDataAdapter da = new MySqlDataAdapter( "SELECT * FROM test", conn);
+			MySqlDataAdapter da = new MySqlDataAdapter( "SELECT * FROM Test", conn);
 			DataTable dt = new DataTable();
 			da.Fill(dt);
 
@@ -462,164 +464,169 @@ namespace MySql.Data.MySqlClient.Tests
 			}
 		}*/
 
+        [Test]
+        public void BitAndDecimal()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (bt1 BIT(2), bt4 BIT(4), bt11 BIT(11), bt23 BIT(23), bt32 BIT(32)) engine=myisam");
+            execSQL("INSERT INTO Test VALUES (2, 3, 120, 240, 1000)");
+            execSQL("INSERT INTO Test VALUES (NULL, NULL, 100, NULL, NULL)");
+
+            string connStr = GetConnectionString(true) + ";treat tiny as boolean=false";
+            using (MySqlConnection c = new MySqlConnection(connStr))
+            {
+                c.Open();
+
+                MySqlCommand cmd = new MySqlCommand("SELECT * FROM Test", c);
+                using (MySqlDataReader reader = cmd.ExecuteReader())
+                {
+                    try
+                    {
+                        Assert.IsTrue(reader.Read());
+                        Assert.AreEqual(2, reader.GetInt32(0));
+                        Assert.AreEqual(3, reader.GetInt32(1));
+                        Assert.AreEqual(120, reader.GetInt32(2));
+                        if (version >= new Version(5, 0))
+                        {
+                            Assert.AreEqual(240, reader.GetInt32(3));
+                            Assert.AreEqual(1000, reader.GetInt32(4));
+                        }
+                        else
+                        {
+                            Assert.AreEqual(127, reader.GetInt32(3));
+                            Assert.AreEqual(127, reader.GetInt32(4));
+                        }
+
+                        Assert.IsTrue(reader.Read());
+                        Assert.IsTrue(reader.IsDBNull(0));
+                        Assert.IsTrue(reader.IsDBNull(1));
+                        Assert.AreEqual(100, reader.GetInt32(2));
+                        Assert.IsTrue(reader.IsDBNull(3));
+                        Assert.IsTrue(reader.IsDBNull(4));
+                    }
+                    catch (Exception ex)
+                    {
+                        Assert.Fail(ex.Message);
+                    }
+                }
+            }
+        }
+
+        /// <summary>
+        /// Bug #10486 MySqlDataAdapter.Update error for decimal column 
+        /// </summary>
+        [Test]
+        public void UpdateDecimalColumns()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id int not null auto_increment primary key, " +
+                "dec1 decimal(10,1))");
+
+            MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", conn);
+            MySqlCommandBuilder cb = new MySqlCommandBuilder(da);
+            DataTable dt = new DataTable();
+            da.Fill(dt);
+            DataRow row = dt.NewRow();
+            row["id"] = DBNull.Value;
+            row["dec1"] = 23.4;
+            dt.Rows.Add(row);
+            da.Update(dt);
+
+            dt.Clear();
+            da.Fill(dt);
+            Assert.AreEqual(1, dt.Rows.Count);
+            Assert.AreEqual(1, dt.Rows[0]["id"]);
+            Assert.AreEqual(23.4, dt.Rows[0]["dec1"]);
+            cb.Dispose();
+        }
+
+        [Test]
+        public void DecimalTests()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (val decimal(10,1))");
+
+            MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES(?dec)", conn);
+            cmd.Parameters.AddWithValue("?dec", (decimal)2.4);
+            Assert.AreEqual(1, cmd.ExecuteNonQuery());
+
+            cmd.Prepare();
+            Assert.AreEqual(1, cmd.ExecuteNonQuery());
+
+            cmd.CommandText = "SELECT * FROM Test";
+            MySqlDataReader reader = null;
+            try
+            {
+                reader = cmd.ExecuteReader();
+                Assert.IsTrue(reader.Read());
+                Assert.IsTrue(reader[0] is Decimal);
+                Assert.AreEqual(2.4, reader[0]);
+
+                Assert.IsTrue(reader.Read());
+                Assert.IsTrue(reader[0] is Decimal);
+                Assert.AreEqual(2.4, reader[0]);
+
+                Assert.IsFalse(reader.Read());
+                Assert.IsFalse(reader.NextResult());
+            }
+            catch (Exception ex)
+            {
+                Assert.Fail(ex.Message);
+            }
+            finally
+            {
+                if (reader != null) reader.Close();
+            }
+        }
+
+        [Test]
+        public void DecimalTests2()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (val decimal(10,1))");
+
+            MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES(?dec)", conn);
+            cmd.Parameters.AddWithValue("?dec", (decimal)2.4);
+            Assert.AreEqual(1, cmd.ExecuteNonQuery());
+
+            cmd.Prepare();
+            Assert.AreEqual(1, cmd.ExecuteNonQuery());
+
+            cmd.CommandText = "SELECT * FROM Test";
+            MySqlDataReader reader = null;
+            try
+            {
+                reader = cmd.ExecuteReader();
+                Assert.IsTrue(reader.Read());
+                Assert.IsTrue(reader[0] is Decimal);
+                Assert.AreEqual(2.4, reader[0]);
+
+                Assert.IsTrue(reader.Read());
+                Assert.IsTrue(reader[0] is Decimal);
+                Assert.AreEqual(2.4, reader[0]);
+
+                Assert.IsFalse(reader.Read());
+                Assert.IsFalse(reader.NextResult());
+            }
+            catch (Exception ex)
+            {
+                Assert.Fail(ex.Message);
+            }
+            finally
+            {
+                if (reader != null) reader.Close();
+            }
+        }
+    
 		[Test]
-		public void BitAndDecimal()
-		{
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (bt1 BIT, bt4 BIT(4), bt11 BIT(11), bt23 BIT(23), bt32 BIT(32)) engine=myisam");
-			execSQL("INSERT INTO test VALUES (1, 2, 120, 240, 1000)");
-			execSQL("INSERT INTO test VALUES (NULL, NULL, 100, NULL, NULL)");
-
-			MySqlCommand cmd = new MySqlCommand("SELECT * FROM test", conn);
-			MySqlDataReader reader = null;
-			try
-			{
-				reader = cmd.ExecuteReader();
-				Assert.IsTrue(reader.Read());
-				Assert.AreEqual(1, reader.GetInt32(0));
-				Assert.AreEqual(2, reader.GetInt32(1));
-				Assert.AreEqual(120, reader.GetInt32(2));
-				if (Is50)
-				{
-					Assert.AreEqual(240, reader.GetInt32(3));
-					Assert.AreEqual(1000, reader.GetInt32(4));
-				}
-				else
-				{
-					Assert.AreEqual(127, reader.GetInt32(3));
-					Assert.AreEqual(127, reader.GetInt32(4));
-				}
-
-				Assert.IsTrue(reader.Read());
-				Assert.IsTrue(reader.IsDBNull(0));
-				Assert.IsTrue(reader.IsDBNull(1));
-				Assert.AreEqual(100, reader.GetInt32(2));
-				Assert.IsTrue(reader.IsDBNull(3));
-				Assert.IsTrue(reader.IsDBNull(4));
-			}
-			catch (Exception ex)
-			{
-				Assert.Fail(ex.Message);
-			}
-			finally
-			{
-				if (reader != null) reader.Close();
-			}
-		}
-
-		/// <summary>
-		/// Bug #10486 MySqlDataAdapter.Update error for decimal column 
-		/// </summary>
-		[Test]
-		public void UpdateDecimalColumns()
-		{
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (id int not null auto_increment primary key, " +
-				"dec1 decimal(10,1))");
-
-			MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM test", conn);
-			MySqlCommandBuilder cb = new MySqlCommandBuilder(da);
-			DataTable dt = new DataTable();
-			da.Fill(dt);
-			DataRow row = dt.NewRow();
-			row["id"] = DBNull.Value;
-			row["dec1"] = 23.4;
-			dt.Rows.Add(row);
-			da.Update(dt);
-
-			dt.Clear();
-			da.Fill(dt);
-			Assert.AreEqual(1, dt.Rows.Count);
-			Assert.AreEqual(1, dt.Rows[0]["id"]);
-			Assert.AreEqual(23.4, dt.Rows[0]["dec1"]);
-		}
-
-		[Test]
-		public void DecimalTests()
-		{
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (val decimal(10,1))");
-
-			MySqlCommand cmd = new MySqlCommand("INSERT INTO test VALUES(?dec)", conn);
-			cmd.Parameters.AddWithValue("?dec", (decimal)2.4);
-			Assert.AreEqual(1, cmd.ExecuteNonQuery());
-
-			cmd.Prepare();
-			Assert.AreEqual(1, cmd.ExecuteNonQuery());
-
-			cmd.CommandText = "SELECT * FROM test";
-			MySqlDataReader reader = null;
-			try
-			{
-				reader = cmd.ExecuteReader();
-				Assert.IsTrue(reader.Read());
-				Assert.IsTrue(reader[0] is Decimal);
-				Assert.AreEqual(2.4, reader[0]);
-
-				Assert.IsTrue(reader.Read());
-				Assert.IsTrue(reader[0] is Decimal);
-				Assert.AreEqual(2.4, reader[0]);
-
-				Assert.IsFalse(reader.Read());
-				Assert.IsFalse(reader.NextResult());
-			}
-			catch (Exception ex)
-			{
-				Assert.Fail(ex.Message);
-			}
-			finally
-			{
-				if (reader != null) reader.Close();
-			}
-		}
-
-		[Test]
-		public void DecimalTests2()
-		{
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (val decimal(10,1))");
-
-			MySqlCommand cmd = new MySqlCommand("INSERT INTO test VALUES(?dec)", conn);
-			cmd.Parameters.AddWithValue("?dec", (decimal)2.4);
-			Assert.AreEqual(1, cmd.ExecuteNonQuery());
-
-			cmd.Prepare();
-			Assert.AreEqual(1, cmd.ExecuteNonQuery());
-
-			cmd.CommandText = "SELECT * FROM test";
-			MySqlDataReader reader = null;
-			try
-			{
-				reader = cmd.ExecuteReader();
-				Assert.IsTrue(reader.Read());
-				Assert.IsTrue(reader[0] is Decimal);
-				Assert.AreEqual(2.4, reader[0]);
-
-				Assert.IsTrue(reader.Read());
-				Assert.IsTrue(reader[0] is Decimal);
-				Assert.AreEqual(2.4, reader[0]);
-
-				Assert.IsFalse(reader.Read());
-				Assert.IsFalse(reader.NextResult());
-			}
-			catch (Exception ex)
-			{
-				Assert.Fail(ex.Message);
-			}
-			finally
-			{
-				if (reader != null) reader.Close();
-			}
-		}
-
-		[Test]
-		[Category("5.0")]
 		public void Bit()
 		{
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (bit1 BIT, bit2 BIT(5), bit3 BIT(10))");
+            if (version < new Version(5, 0)) return;
 
-			MySqlCommand cmd = new MySqlCommand("INSERT INTO test VALUES (?b1, ?b2, ?b3)", conn);
+            execSQL("DROP TABLE IF EXISTS Test");
+			execSQL("CREATE TABLE Test (bit1 BIT, bit2 BIT(5), bit3 BIT(10))");
+
+			MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES (?b1, ?b2, ?b3)", conn);
 			cmd.Parameters.Add(new MySqlParameter("?b1", MySqlDbType.Bit));
 			cmd.Parameters.Add(new MySqlParameter("?b2", MySqlDbType.Bit));
 			cmd.Parameters.Add(new MySqlParameter("?b3", MySqlDbType.Bit));
@@ -632,7 +639,7 @@ namespace MySql.Data.MySqlClient.Tests
 			MySqlDataReader reader = null;
 			try 
 			{
-				cmd.CommandText = "SELECT * FROM test";
+				cmd.CommandText = "SELECT * FROM Test";
 				cmd.Prepare();
 				reader = cmd.ExecuteReader();
 				Assert.IsTrue(reader.Read());
@@ -652,7 +659,7 @@ namespace MySql.Data.MySqlClient.Tests
 
 		/// <summary>
 		/// Bug #17375 CommandBuilder ignores Unsigned flag at Parameter creation 
-		/// Bug #15274 Use MySqlDbType.UInt32, throwed exception 'Only byte arrays can be serialize' 
+        /// Bug #15274 Use MySqlDbType.UInt32, throwed exception 'Only byte arrays can be serialize' 
 		/// </summary>
 		[Test]
 		public void UnsignedTypes()
@@ -678,86 +685,189 @@ namespace MySql.Data.MySqlClient.Tests
 			row["b"] = 135;
 			row.EndEdit();
 			da.Update(dv.Table);
+            cb.Dispose();
 
 			execSQL("DROP TABLE IF EXISTS Test");
 			execSQL("CREATE TABLE Test (b MEDIUMINT UNSIGNED PRIMARY KEY)");
-			execSQL("INSERT INTO Test VALUES(20)");
-			MySqlCommand cmd = new MySqlCommand("SELECT * FROM Test WHERE (b > ?id)", conn);
-			cmd.Parameters.Add("?id", MySqlDbType.UInt16).Value = 10;
-			MySqlDataReader dr = null;
-			try
-			{
-				dr = cmd.ExecuteReader();
-				dr.Read();
-				Assert.AreEqual(20, dr.GetUInt16(0));
-			}
-			catch (Exception ex)
-			{
-				Assert.Fail(ex.Message);
-			}
-			finally
-			{
-				if (dr != null)
-					dr.Close();
-			}
+            execSQL("INSERT INTO Test VALUES(20)");
+            MySqlCommand cmd = new MySqlCommand("SELECT * FROM Test WHERE (b > ?id)", conn);
+            cmd.Parameters.Add("?id", MySqlDbType.UInt16).Value = 10;
+            MySqlDataReader dr = null;
+            try
+            {
+                dr = cmd.ExecuteReader();
+                dr.Read();
+                Assert.AreEqual(20, dr.GetUInt16(0));
+            }
+            catch (Exception ex)
+            {
+                Assert.Fail(ex.Message);
+            }
+            finally
+            {
+                if (dr != null)
+                    dr.Close();
+            }
 		}
 
-		/// <summary>
-		/// Bug #25912 selecting negative time values gets wrong results 
-		/// </summary>
-		[Test]
-		public void TestNegativeTime()
-		{
-			execSQL("DROP TABLE IF EXISTS test");
-			execSQL("CREATE TABLE test (t time)");
-			execSQL("INSERT INTO test SET T='-07:24:00'");
+        /// <summary>
+        /// Bug #25912 selecting negative time values gets wrong results 
+        /// </summary>
+        [Test]
+        public void TestNegativeTime()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (t time)");
+            execSQL("INSERT INTO Test SET T='-07:24:00'");
 
-			MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM test", conn);
-			DataTable dt = new DataTable();
-			da.Fill(dt);
+            MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", conn);
+            DataTable dt = new DataTable();
+            da.Fill(dt);
 
-			TimeSpan ts = (TimeSpan)dt.Rows[0]["t"];
-			Assert.AreEqual(-7, ts.Hours);
-			Assert.AreEqual(-24, ts.Minutes);
-			Assert.AreEqual(0, ts.Seconds);
-		}
+            TimeSpan ts = (TimeSpan)dt.Rows[0]["t"];
+            Assert.AreEqual(-7, ts.Hours);
+            Assert.AreEqual(-24, ts.Minutes);
+            Assert.AreEqual(0, ts.Seconds);
+        }
 
-		/// <summary>
-		/// Bug #25605 BINARY and VARBINARY is returned as a string 
-		/// </summary>
-		[Test]
-		public void BinaryAndVarBinary()
-		{
-			// TODO: uncomment when working
-			return;
+        /// <summary>
+        /// Bug #25605 BINARY and VARBINARY is returned as a string 
+        /// </summary>
+        [Test]
+        public void BinaryAndVarBinary()
+        {
+            MySqlCommand cmd = new MySqlCommand("SELECT BINARY 'something' AS BinaryData", conn);
+            using (MySqlDataReader reader = cmd.ExecuteReader())
+            {
+                reader.Read();
+                byte[] buffer = new byte[2];
+                long read = reader.GetBytes(0, 0, buffer, 0, 2);
+                Assert.AreEqual('s', buffer[0]);
+                Assert.AreEqual('o', buffer[1]);
+                Assert.AreEqual(2, read);
 
-			MySqlCommand cmd = new MySqlCommand("SELECT BINARY 'something' AS BinaryData", conn);
-			using (MySqlDataReader reader = cmd.ExecuteReader())
-			{
-				reader.Read();
-				byte[] buffer = new byte[2];
-				long read = reader.GetBytes(0, 0, buffer, 0, 2);
-				Assert.AreEqual('s', buffer[0]);
-				Assert.AreEqual('o', buffer[1]);
-				Assert.AreEqual(2, read);
+                string s = reader.GetString(0);
+                Assert.AreEqual("something", s);
+            }
+        }
 
-				string s = reader.GetString(0);
-				Assert.AreEqual("something", s);
-			}
-		}
+        [Test]
+        public void NumericAsBinary()
+        {
+            MySqlCommand cmd = new MySqlCommand("SELECT IFNULL(NULL,0) AS MyServerID", conn);
+            using (MySqlDataReader reader = cmd.ExecuteReader())
+            {
+                reader.Read();
+                Assert.AreEqual("BIGINT", reader.GetDataTypeName(0));
+                Assert.AreEqual(typeof(Int64), reader.GetFieldType(0));
+                Assert.AreEqual("System.Int64", reader.GetValue(0).GetType().FullName);
+                Assert.AreEqual(0, reader.GetValue(0));
+            }
+        }
 
-		[Test]
-		public void NumericAsBinary()
-		{
-			MySqlCommand cmd = new MySqlCommand("SELECT IFNULL(NULL,0) AS MyServerID", conn);
-			using (MySqlDataReader reader = cmd.ExecuteReader())
-			{
-				reader.Read();
-				Assert.AreEqual("BIGINT", reader.GetDataTypeName(0));
-				Assert.AreEqual(typeof(Int64), reader.GetFieldType(0));
-				Assert.AreEqual("System.Int64", reader.GetValue(0).GetType().FullName);
-				Assert.AreEqual(0, reader.GetValue(0));
-			}
-		}
+        [Test]
+        public void BinaryTypes()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL(@"CREATE TABLE Test (c1 VARCHAR(20), c2 VARBINARY(20),
+                c3 TEXT, c4 BLOB, c6 VARCHAR(20) CHARACTER SET BINARY)");
+
+            MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", conn);
+            DataTable dt = new DataTable();
+            da.Fill(dt);
+            Assert.AreEqual(typeof(String), dt.Columns[0].DataType);
+            Assert.AreEqual(typeof(byte[]), dt.Columns[1].DataType);
+            Assert.AreEqual(typeof(String), dt.Columns[2].DataType);
+            Assert.AreEqual(typeof(byte[]), dt.Columns[3].DataType);
+            Assert.AreEqual(typeof(byte[]), dt.Columns[4].DataType);
+        }
+
+        [Test]
+        public void ShowColumns()
+        {
+            if (version < new Version(5, 0)) return;
+
+            MySqlDataAdapter da = new MySqlDataAdapter(
+                @"SELECT TRIM(TRAILING ' unsigned' FROM 
+                  TRIM(TRAILING ' zerofill' FROM COLUMN_TYPE)) AS MYSQL_TYPE, 
+                  IF(COLUMN_DEFAULT IS NULL, NULL, 
+                  IF(ASCII(COLUMN_DEFAULT) = 1 OR COLUMN_DEFAULT = '1', 1, 0))
+                  AS TRUE_DEFAULT FROM INFORMATION_SCHEMA.COLUMNS
+                  WHERE TABLE_SCHEMA='test' AND TABLE_NAME='test'", conn);
+            DataTable dt = new DataTable();
+            da.Fill(dt);
+
+            Assert.AreEqual(typeof(string), dt.Columns[0].DataType);
+            Assert.AreEqual(typeof(Int64), dt.Columns[1].DataType);
+        }
+
+        [Test]
+        public void RespectBinaryFlag()
+        {
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (col1 VARBINARY(20), col2 BLOB)");
+
+            string connStr = GetConnectionString(true) + ";respect binary flags=false";
+
+            using (MySqlConnection c = new MySqlConnection(connStr))
+            {
+                c.Open();
+                MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", c);
+                DataTable dt = new DataTable();
+                da.Fill(dt);
+                Assert.IsTrue(dt.Columns[0].DataType == typeof(System.String));
+                Assert.IsTrue(dt.Columns[1].DataType == typeof(System.Byte[]));
+            }
+        }
+
+        /// <summary>
+        /// Bug #27959 Bool datatype is not returned as System.Boolean by MySqlDataAdapter 
+        /// </summary>
+        [Test]
+        public void Boolean()
+        {
+            if (version < new Version(5, 0)) return;
+
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id INT, `on` BOOLEAN, v TINYINT(2))");
+            execSQL("INSERT INTO Test VALUES (1,1,1), (2,0,0)");
+
+            MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", conn);
+            DataTable dt = new DataTable();
+            da.Fill(dt);
+            Assert.AreEqual(typeof(Boolean), dt.Columns[1].DataType);
+            Assert.AreEqual(typeof(SByte), dt.Columns[2].DataType);
+            Assert.AreEqual(true, dt.Rows[0][1]);
+            Assert.AreEqual(false, dt.Rows[1][1]);
+            Assert.AreEqual(1, dt.Rows[0][2]);
+            Assert.AreEqual(0, dt.Rows[1][2]);
+        }
+
+        [Test]
+        public void Binary16AsGuid()
+        {
+            if (version < new Version(5, 0)) return;
+
+            execSQL("DROP TABLE IF EXISTS Test");
+            execSQL("CREATE TABLE Test (id INT, g BINARY(16), c VARBINARY(16), c1 BINARY(17))");
+
+            Guid g = Guid.NewGuid();
+            byte[] bytes = g.ToByteArray();
+
+            MySqlCommand cmd = new MySqlCommand("INSERT INTO Test VALUES (1, @g, @c, @c1)", conn);
+            cmd.Parameters.AddWithValue("@g", bytes);
+            cmd.Parameters.AddWithValue("@c", bytes);
+            cmd.Parameters.AddWithValue("@c1", g.ToString());
+            cmd.ExecuteNonQuery();
+
+            MySqlDataAdapter da = new MySqlDataAdapter("SELECT * FROM Test", conn);
+            DataTable dt = new DataTable();
+            da.Fill(dt);
+            Assert.IsTrue(dt.Rows[0][1] is Guid);
+            Assert.IsTrue(dt.Rows[0][2] is byte[]);
+            Assert.IsTrue(dt.Rows[0][3] is byte[]);
+
+            Assert.AreEqual(g, dt.Rows[0][1]);
+        }
 	}
 }
