@@ -21,11 +21,11 @@
 using System;
 using System.Data;
 using MySql.Data.MySqlClient;
-using NUnit.Framework;
+using MbUnit.Framework;
 
 namespace MySql.Data.MySqlClient.Tests
 {
-	[NUnit.Framework.TestFixture]
+	[MbUnit.Framework.TestFixture]
 	public class GetSchemaTests : BaseTest
 	{
 		[Test]
@@ -46,7 +46,7 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("IndexColumns", dt.Rows[10][0]);
 			Assert.AreEqual("Indexes", dt.Rows[11][0]);
 
-			if (Is50)
+			if (version >= new Version(5, 0))
 			{
 				Assert.AreEqual("Views", dt.Rows[12][0]);
 				Assert.AreEqual("ViewColumns", dt.Rows[13][0]);
@@ -224,10 +224,11 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("TINYINT", dt.Rows[3]["DATA_TYPE"].ToString().ToUpper());
 		}
 
-		[Category("5.0")]
 		[Test]
 		public void Procedures()
 		{
+			if (version < new Version(5, 0)) return;
+
 			execSQL("DROP PROCEDURE IF EXISTS spTest");
 			execSQL("CREATE PROCEDURE spTest (id int) BEGIN SELECT 1; END");
 
@@ -240,10 +241,11 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("spTest", dt.Rows[0][3]);
 		}
 
-		[Category("5.0")]
 		[Test]
 		public void Functions()
 		{
+			if (version < new Version(5, 0)) return;
+
 			execSQL("DROP FUNCTION IF EXISTS spFunc");
 			execSQL("CREATE FUNCTION spFunc (id int) RETURNS INT BEGIN RETURN 1; END");
 
@@ -256,10 +258,11 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("spFunc", dt.Rows[0][3]);
 		}
 
-		[Category("5.0")]
 		[Test]
 		public void ProcedureParameters()
 		{
+			if (version < new Version(5, 0)) return;
+
 			execSQL("DROP PROCEDURE IF EXISTS spTest");
 			execSQL("CREATE PROCEDURE spTest (id int, name varchar(50)) BEGIN SELECT 1; END");
 
@@ -271,18 +274,18 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("Procedure Parameters", dt.TableName);
 			Assert.AreEqual(database0.ToLower(), dt.Rows[0]["ROUTINE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("sptest", dt.Rows[0]["ROUTINE_NAME"].ToString().ToLower());
-			Assert.AreEqual("?id", dt.Rows[0]["PARAMETER_NAME"].ToString().ToLower());
+			Assert.AreEqual("@id", dt.Rows[0]["PARAMETER_NAME"].ToString().ToLower());
 			Assert.AreEqual(1, dt.Rows[0]["ORDINAL_POSITION"]);
 			Assert.AreEqual("IN", dt.Rows[0]["PARAMETER_MODE"]);
 			Assert.AreEqual("NO", dt.Rows[0]["IS_RESULT"]);
 
-			restrictions[4] = "?name";
+			restrictions[4] = "@name";
 			dt.Clear();
 			dt = conn.GetSchema("Procedure Parameters", restrictions);
 			Assert.AreEqual(1, dt.Rows.Count);
 			Assert.AreEqual(database0.ToLower(), dt.Rows[0]["ROUTINE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("sptest", dt.Rows[0]["ROUTINE_NAME"].ToString().ToLower());
-			Assert.AreEqual("?name", dt.Rows[0]["PARAMETER_NAME"].ToString().ToLower());
+			Assert.AreEqual("@name", dt.Rows[0]["PARAMETER_NAME"].ToString().ToLower());
 			Assert.AreEqual(2, dt.Rows[0]["ORDINAL_POSITION"]);
 			Assert.AreEqual("IN", dt.Rows[0]["PARAMETER_MODE"]);
 			Assert.AreEqual("NO", dt.Rows[0]["IS_RESULT"]);
@@ -298,7 +301,7 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("Procedure Parameters", dt.TableName);
 			Assert.AreEqual(database0.ToLower(), dt.Rows[0]["ROUTINE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("spfunc", dt.Rows[0]["ROUTINE_NAME"].ToString().ToLower());
-			Assert.AreEqual("?id", dt.Rows[0]["PARAMETER_NAME"].ToString().ToLower());
+			Assert.AreEqual("@id", dt.Rows[0]["PARAMETER_NAME"].ToString().ToLower());
 			Assert.AreEqual(1, dt.Rows[0]["ORDINAL_POSITION"]);
 			Assert.AreEqual("IN", dt.Rows[0]["PARAMETER_MODE"]);
 			Assert.AreEqual("NO", dt.Rows[0]["IS_RESULT"]);
@@ -309,10 +312,11 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("YES", dt.Rows[1]["IS_RESULT"]);
 		}
 
-		[Category("5.0")]
 		[Test]
 		public void Indexes()
 		{
+			if (version < new Version(5, 0)) return;
+
 			execSQL("DROP TABLE IF EXISTS test");
 			execSQL("CREATE TABLE test (id int, PRIMARY KEY(id))");
 			string[] restrictions = new string[4];
@@ -385,10 +389,11 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual(2, dt.Rows[1]["ORDINAL_POSITION"]);
 		}
 
-		[Category("5.0")]
 		[Test]
 		public void Views()
 		{
+			if (version < new Version(5, 0)) return;
+
 			execSQL("DROP VIEW IF EXISTS vw");
 			execSQL("CREATE VIEW vw AS SELECT Now() as theTime");
 
@@ -401,10 +406,11 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("vw", dt.Rows[0]["TABLE_NAME"]);
 		}
 
-		[Category("5.0")]
 		[Test]
 		public void ViewColumns()
 		{
+			if (version < new Version(5, 0)) return;
+
 			execSQL("DROP VIEW IF EXISTS vw");
 			execSQL("CREATE VIEW vw AS SELECT Now() as theTime");
 
@@ -420,9 +426,10 @@ namespace MySql.Data.MySqlClient.Tests
 		}
 
 		[Test]
-		[Category("5.0")]
 		public void SingleProcedureParameters()
 		{
+			if (version < new Version(5, 0)) return;
+
 			execSQL("DROP PROCEDURE IF EXISTS spTest");
 			execSQL("CREATE PROCEDURE spTest(id int, IN id2 INT(11), " +
 				"INOUT io1 VARCHAR(20), OUT out1 FLOAT) BEGIN END");
@@ -453,28 +460,28 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual("spTest", parameters.Rows[3][2]);
 
 			Assert.AreEqual("PROCEDURE", parameters.Rows[0][3]);
-			Assert.AreEqual("?id", parameters.Rows[0][4]);
+			Assert.AreEqual("@id", parameters.Rows[0][4]);
 			Assert.AreEqual(1, parameters.Rows[0][5]);
 			Assert.AreEqual("IN", parameters.Rows[0][6]);
 			Assert.AreEqual("NO", parameters.Rows[0][7]);
 			Assert.AreEqual("INT", parameters.Rows[0][8].ToString().ToUpper());
 
 			Assert.AreEqual("PROCEDURE", parameters.Rows[1][3]);
-			Assert.AreEqual("?id2", parameters.Rows[1][4]);
+			Assert.AreEqual("@id2", parameters.Rows[1][4]);
 			Assert.AreEqual(2, parameters.Rows[1][5]);
 			Assert.AreEqual("IN", parameters.Rows[1][6]);
 			Assert.AreEqual("NO", parameters.Rows[1][7]);
 			Assert.AreEqual("INT", parameters.Rows[1][8].ToString().ToUpper());
 
 			Assert.AreEqual("PROCEDURE", parameters.Rows[2][3]);
-			Assert.AreEqual("?io1", parameters.Rows[2][4]);
+			Assert.AreEqual("@io1", parameters.Rows[2][4]);
 			Assert.AreEqual(3, parameters.Rows[2][5]);
 			Assert.AreEqual("INOUT", parameters.Rows[2][6]);
 			Assert.AreEqual("NO", parameters.Rows[2][7]);
 			Assert.AreEqual("VARCHAR", parameters.Rows[2][8].ToString().ToUpper());
 
 			Assert.AreEqual("PROCEDURE", parameters.Rows[3][3]);
-			Assert.AreEqual("?out1", parameters.Rows[3][4]);
+			Assert.AreEqual("@out1", parameters.Rows[3][4]);
 			Assert.AreEqual(4, parameters.Rows[3][5]);
 			Assert.AreEqual("OUT", parameters.Rows[3][6]);
 			Assert.AreEqual("NO", parameters.Rows[3][7]);
@@ -502,11 +509,8 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual(DBNull.Value, row["TABLE_CATALOG"]);
 			Assert.AreEqual(database0.ToLower(), row["TABLE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("child", row["TABLE_NAME"]);
-			Assert.AreEqual("parent_id", row["COLUMN_NAME"]);
-			Assert.AreEqual(0, row["ORDINAL_POSITION"]);
 			Assert.AreEqual(database0.ToLower(), row["REFERENCED_TABLE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("parent", row["REFERENCED_TABLE_NAME"]);
-			Assert.AreEqual("id", row["REFERENCED_COLUMN_NAME"]);
 		}
 
 		/// <summary>
@@ -530,7 +534,7 @@ namespace MySql.Data.MySqlClient.Tests
 
 			try 
 			{
-				DataTable dt = conn.GetSchema("Foreign Keys");
+				conn.GetSchema("Foreign Keys");
 			}
 			catch (Exception ex)
 			{
@@ -559,7 +563,7 @@ namespace MySql.Data.MySqlClient.Tests
 			restrictions[1] = database0;
 			restrictions[2] = "product_order";
 			DataTable dt = conn.GetSchema("Foreign Keys", restrictions);
-			Assert.AreEqual(3, dt.Rows.Count);
+			Assert.AreEqual(2, dt.Rows.Count);
 			DataRow row = dt.Rows[0];
 			Assert.AreEqual(DBNull.Value, row["CONSTRAINT_CATALOG"]);
 			Assert.AreEqual(database0.ToLower(), row["CONSTRAINT_SCHEMA"].ToString().ToLower());
@@ -567,43 +571,25 @@ namespace MySql.Data.MySqlClient.Tests
 			Assert.AreEqual(DBNull.Value, row["TABLE_CATALOG"]);
 			Assert.AreEqual(database0.ToLower(), row["TABLE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("product_order", row["TABLE_NAME"]);
-			Assert.AreEqual("product_category", row["COLUMN_NAME"]);
-			Assert.AreEqual(0, row["ORDINAL_POSITION"]);
 			Assert.AreEqual(database0.ToLower(), row["REFERENCED_TABLE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("product", row["REFERENCED_TABLE_NAME"]);
-			Assert.AreEqual("category", row["REFERENCED_COLUMN_NAME"]);
 
 			row = dt.Rows[1];
 			Assert.AreEqual(DBNull.Value, row["CONSTRAINT_CATALOG"]);
 			Assert.AreEqual(database0.ToLower(), row["CONSTRAINT_SCHEMA"].ToString().ToLower());
-			Assert.AreEqual("product_order_ibfk_1", row["CONSTRAINT_NAME"]);
-			Assert.AreEqual(DBNull.Value, row["TABLE_CATALOG"]);
-			Assert.AreEqual(database0.ToLower(), row["TABLE_SCHEMA"].ToString().ToLower());
-			Assert.AreEqual("product_order", row["TABLE_NAME"]);
-			Assert.AreEqual("product_id", row["COLUMN_NAME"]);
-			Assert.AreEqual(1, row["ORDINAL_POSITION"]);
-			Assert.AreEqual(database0.ToLower(), row["REFERENCED_TABLE_SCHEMA"].ToString().ToLower());
-			Assert.AreEqual("product", row["REFERENCED_TABLE_NAME"]);
-			Assert.AreEqual("id", row["REFERENCED_COLUMN_NAME"]);
-
-			row = dt.Rows[2];
-			Assert.AreEqual(DBNull.Value, row["CONSTRAINT_CATALOG"]);
-			Assert.AreEqual(database0.ToLower(), row["CONSTRAINT_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("product_order_ibfk_2", row["CONSTRAINT_NAME"]);
 			Assert.AreEqual(DBNull.Value, row["TABLE_CATALOG"]);
 			Assert.AreEqual(database0.ToLower(), row["TABLE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("product_order", row["TABLE_NAME"]);
-			Assert.AreEqual("customer_id", row["COLUMN_NAME"]);
-			Assert.AreEqual(0, row["ORDINAL_POSITION"]);
 			Assert.AreEqual(database0.ToLower(), row["REFERENCED_TABLE_SCHEMA"].ToString().ToLower());
 			Assert.AreEqual("customer", row["REFERENCED_TABLE_NAME"]);
-			Assert.AreEqual("id", row["REFERENCED_COLUMN_NAME"]);
 		}
 
-		[Category("5.0")]
 		[Test]
 		public void Triggers()
 		{
+			if (version < new Version(5, 0)) return;
+
 			try
 			{
 				suExecSQL("DROP TRIGGER trigger1");
