@@ -44,7 +44,13 @@
  
  u s i n g   M y S q l . D a t a . M y S q l C l i e n t ; 
  
- u s i n g   N U n i t . F r a m e w o r k ; 
+ u s i n g   M b U n i t . F r a m e w o r k ; 
+ 
+ u s i n g   S y s t e m . R e f l e c t i o n ; 
+ 
+ u s i n g   S y s t e m . C o l l e c t i o n s ; 
+ 
+ u s i n g   S y s t e m . C o l l e c t i o n s . G e n e r i c ; 
  
  
  
@@ -66,7 +72,7 @@
  
  	 	 [ T e s t F i x t u r e S e t U p ] 
  
- 	 	 p r o t e c t e d   o v e r r i d e   v o i d   F i x t u r e S e t u p ( ) 
+ 	 	 p u b l i c   o v e r r i d e   v o i d   F i x t u r e S e t u p ( ) 
  
  	 	 { 
  
@@ -80,7 +86,7 @@
  
  	 	 [ T e s t ] 
  
- 	 	 p u b l i c   v o i d   C o n n e c t i o n ( )   
+ 	 	 p u b l i c   v o i d   C o n n e c t i o n ( ) 
  
  	 	 { 
  
@@ -88,7 +94,7 @@
  
  
  
- 	 	 	 M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n (   c o n n S t r   ) ; 
+ 	 	 	 M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
  
  	 	 	 c . O p e n ( ) ; 
  
@@ -96,11 +102,11 @@
  
  	 	 	 c . C l o s e ( ) ; 
  
- 	 	 	 
+ 
  
  	 	 	 / /   f i r s t   t e s t   t h a t   o n l y   a   s i n g l e   c o n n e c t i o n   g e t ' s   u s e d 
  
- 	 	 	 f o r   ( i n t   i = 0 ;   i   <   1 0 ;   i + + )   
+ 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   1 0 ;   i + + ) 
  
  	 	 	 { 
  
@@ -128,7 +134,7 @@
  
  	 	 	 M y S q l C o n n e c t i o n [ ]   c o n n A r r a y   =   n e w   M y S q l C o n n e c t i o n [ 1 0 ] ; 
  
- 	 	 	 f o r   ( i n t   i = 0 ;   i   <   c o n n A r r a y . L e n g t h ;   i + + )   
+ 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   c o n n A r r a y . L e n g t h ;   i + + ) 
  
  	 	 	 { 
  
@@ -142,11 +148,11 @@
  
  	 	 	 / /   n o w   m a k e   s u r e   a l l   t h e   s e r v e r   i d s   a r e   d i f f e r e n t 
  
- 	 	 	 f o r   ( i n t   i = 0 ;   i   <   c o n n A r r a y . L e n g t h ;   i + + )   
+ 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   c o n n A r r a y . L e n g t h ;   i + + ) 
  
  	 	 	 { 
  
- 	 	 	 	 f o r   ( i n t   j = 0 ;   j   <   c o n n A r r a y . L e n g t h ;   j + + ) 
+ 	 	 	 	 f o r   ( i n t   j   =   0 ;   j   <   c o n n A r r a y . L e n g t h ;   j + + ) 
  
  	 	 	 	 { 
  
@@ -160,7 +166,7 @@
  
  
  
- 	 	 	 f o r   ( i n t   i = 0 ;   i   <   c o n n A r r a y . L e n g t h ;   i + + ) 
+ 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   c o n n A r r a y . L e n g t h ;   i + + ) 
  
  	 	 	 { 
  
@@ -180,7 +186,7 @@
  
  	 	 { 
  
- 	 	 	 t r y   
+ 	 	 	 t r y 
  
  	 	 	 { 
  
@@ -234,7 +240,7 @@
  
  	 	 [ T e s t ] 
  
- 	 	 p u b l i c   v o i d   R e c l a i m B r o k e n C o n n e c t i o n ( )   
+ 	 	 p u b l i c   v o i d   R e c l a i m B r o k e n C o n n e c t i o n ( ) 
  
  	 	 { 
  
@@ -254,7 +260,7 @@
  
  	 	 	 / /   n o w   a t t e m p t i n g   t o   o p e n   a   c o n n e c t i o n   s h o u l d   f a i l 
  
- 	 	 	 t r y   
+ 	 	 	 t r y 
  
  	 	 	 { 
  
@@ -278,7 +284,7 @@
  
  	 	 	 / /   n o w   w e   d o   s o m e t h i n g   o n   t h e   f i r s t   c o n n e c t i o n 
  
- 	 	 	 t r y   
+ 	 	 	 t r y 
  
  	 	 	 { 
  
@@ -294,7 +300,7 @@
  
  	 	 	 / /   O p e n i n g   a   c o n n e c t i o n   n o w   s h o u l d   w o r k 
  
- 	 	 	 t r y   
+ 	 	 	 t r y 
  
  	 	 	 { 
  
@@ -306,9 +312,9 @@
  
  	 	 	 } 
  
- 	 	 	 c a t c h   ( E x c e p t i o n   e x )   
+ 	 	 	 c a t c h   ( E x c e p t i o n   e x ) 
  
- 	 	 	 {   
+ 	 	 	 { 
  
  	 	 	 	 A s s e r t . F a i l ( e x . M e s s a g e ) ; 
  
@@ -432,6 +438,10 @@
  
  
  
+ # i f   ! C F 
+ 
+ 
+ 
  	 	 p r i v a t e   v o i d   P o o l i n g W o r k e r ( o b j e c t   c n ) 
  
  	 	 { 
@@ -500,7 +510,11 @@
  
  
  
- 	 	 [ C a t e g o r y ( " 5 . 0 " ) ] 
+ # e n d i f 
+ 
+ 
+ 
+ 
  
  	 	 [ T e s t ] 
  
@@ -508,6 +522,10 @@
  
  	 	 { 
  
+ 	 	 	 i f   ( v e r s i o n   <   n e w   V e r s i o n ( 5 ,   0 ) )   r e t u r n ; 
+ 
+ 
+ 
  	 	 	 e x e c S Q L ( " D R O P   T A B L E   I F   E X I S T S   t e s t " ) ; 
  
  	 	 	 e x e c S Q L ( " D R O P   P R O C E D U R E   I F   E X I S T S   s p T e s t " ) ; 
@@ -546,6 +564,8 @@
  
  	 	 	 	 	 	 M y S q l C o m m a n d   r e c c m d   =   n e w   M y S q l C o m m a n d ( " s p T e s t " ,   c o n ) ; 
  
+                                                 r e c c m d . C o m m a n d T i m e o u t   =   0 ; 
+ 
  	 	 	 	 	 	 r e c c m d . C o m m a n d T y p e   =   C o m m a n d T y p e . S t o r e d P r o c e d u r e ; 
  
  	 	 	 	 	 	 M y S q l P a r a m e t e r   p a r   =   n e w   M y S q l P a r a m e t e r ( " t h e i d " ,   M y S q l D b T y p e . S t r i n g ) ; 
@@ -602,33 +622,201 @@
  
  	 	 	 / / T O D O :   r e f a c t o r   t e s t   s u i t e   t o   s u p p o r t   s t a r t i n g / s t o p p i n g   s e r v i c e s 
  
- / *                         s t r i n g   c o n n S t r   =   " s e r v e r = l o c a l h o s t ; u i d = r o o t ; d a t a b a s e = t e s t ; p o o l i n g = t r u e ; c o n n e c t   t i m e o u t = 6 ;   m a x   p o o l   s i z e   =   6 " ; 
+ 	 	 	 / *                         s t r i n g   c o n n S t r   =   " s e r v e r = l o c a l h o s t ; u i d = r o o t ; d a t a b a s e = t e s t ; p o o l i n g = t r u e ; c o n n e c t   t i m e o u t = 6 ;   m a x   p o o l   s i z e   =   6 " ; 
  
- 	 	 	 M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
+ 	 	 	 	 	 	 M y S q l C o n n e c t i o n   c   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
  
- 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   6 ;   i + + ) 
+ 	 	 	 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   6 ;   i + + ) 
+ 
+ 	 	 	 	 	 	 { 
+ 
+ 	 	 	 	 	 	 	 t r y 
+ 
+ 	 	 	 	 	 	 	 { 
+ 
+ 	 	 	 	 	 	 	 	 c . O p e n ( ) ; 
+ 
+ 	 	 	 	 	 	 	 } 
+ 
+ 	 	 	 	 	 	 	 c a t c h   ( E x c e p t i o n   e x ) 
+ 
+ 	 	 	 	 	 	 	 { 
+ 
+ 	 	 	 	 	 	 	 } 
+ 
+ 	 	 	 	 	 	 } 
+ 
+ 	 	 	 	 	 	 c . O p e n ( ) ; 
+ 
+ 	 	 	 	 	 	 c . C l o s e ( ) ; * / 
+ 
+ 	 	 } 
+ 
+ 
+ 
+ 	 	 [ T e s t ] 
+ 
+ 	 	 p u b l i c   v o i d   C l e a r P o o l ( ) 
+ 
+ 	 	 { 
+ 
+ 	 	 	 s t r i n g   c o n n S t r   =   G e t C o n n e c t i o n S t r i n g ( t r u e )   +   " ;   m i n   p o o l   s i z e = 1 0 " ; 
+ 
+ 	 	 	 M y S q l C o n n e c t i o n [ ]   c o n n e c t i o n s   =   n e w   M y S q l C o n n e c t i o n [ 1 0 ] ; 
+ 
+ 	 	 	 c o n n e c t i o n s [ 0 ]   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
+ 
+ 	 	 	 c o n n e c t i o n s [ 0 ] . O p e n ( ) ; 
+ 
+ 
+ 
+                         s t r i n g   a s s e m b l y N a m e   =   t y p e o f ( M y S q l C o n n e c t i o n ) . A s s e m b l y . F u l l N a m e ; 
+ 
+                         s t r i n g   p m N a m e   =   S t r i n g . F o r m a t ( " M y S q l . D a t a . M y S q l C l i e n t . M y S q l P o o l M a n a g e r ,   { 0 } " ,   a s s e m b l y N a m e ) ; 
+ 
+ 
+ 
+                         T y p e   p o o l M a n a g e r   =   T y p e . G e t T y p e ( p m N a m e ,   f a l s e ) ; 
+ 
+                         F i e l d I n f o   p o o l M a n a g e r H a s h T a b l e   =   p o o l M a n a g e r . G e t F i e l d ( " p o o l s " , 
+ 
+                                 B i n d i n g F l a g s . N o n P u b l i c   |   B i n d i n g F l a g s . S t a t i c   |   B i n d i n g F l a g s . I n s t a n c e ) ; 
+ 
+                         H a s h t a b l e   p o o l H a s h   =   ( H a s h t a b l e ) p o o l M a n a g e r H a s h T a b l e . G e t V a l u e ( n u l l ) ; 
+ 
+                         F i e l d I n f o   c l e a r i n g P o o l s F I   =   p o o l M a n a g e r . G e t F i e l d ( " c l e a r i n g P o o l s " , 
+ 
+                                 B i n d i n g F l a g s . N o n P u b l i c   |   B i n d i n g F l a g s . S t a t i c   |   B i n d i n g F l a g s . I n s t a n c e ) ; 
+ 
+                         I C o l l e c t i o n   c l e a r i n g P o o l s   =   ( I C o l l e c t i o n ) c l e a r i n g P o o l s F I . G e t V a l u e ( n u l l ) ; 
+ 
+ 
+ 
+                         / /   n o w   w e   n e e d   t o   i n v e s t i g a t e 
+ 
+                         s t r i n g   p o o l N a m e   =   S t r i n g . F o r m a t ( " M y S q l . D a t a . M y S q l C l i e n t . M y S q l P o o l ,   { 0 } " ,   a s s e m b l y N a m e ) ; 
+ 
+                         T y p e   p o o l T y p e   =   T y p e . G e t T y p e ( p o o l N a m e ,   f a l s e ) ; 
+ 
+                         
+ 
+                         F i e l d I n f o   i n U s e P o o l   =   p o o l T y p e . G e t F i e l d ( " i n U s e P o o l " ,   B i n d i n g F l a g s . N o n P u b l i c   |   B i n d i n g F l a g s . I n s t a n c e ) ; 
+ 
+                         I C o l l e c t i o n   i n U s e L i s t   =   ( I C o l l e c t i o n ) i n U s e P o o l . G e t V a l u e ( p o o l H a s h [ c o n n S t r ] ) ; 
+ 
+                         A s s e r t . A r e E q u a l ( 1 ,   i n U s e L i s t . C o u n t ) ; 
+ 
+ 
+ 
+                         F i e l d I n f o   i d l e P o o l   =   p o o l T y p e . G e t F i e l d ( " i d l e P o o l " ,   B i n d i n g F l a g s . N o n P u b l i c   |   B i n d i n g F l a g s . I n s t a n c e ) ; 
+ 
+                         I C o l l e c t i o n   i d l e L i s t   =   ( I C o l l e c t i o n ) i d l e P o o l . G e t V a l u e ( p o o l H a s h [ c o n n S t r ] ) ; 
+ 
+                         A s s e r t . A r e E q u a l ( 9 ,   i d l e L i s t . C o u n t ) ; 
+ 
+ 
+ 
+ 	 	 	 / /   n o w   o p e n   4   m o r e   o f   t h e s e .     N o w   w e   s h o u d l   h a v e   5   o p e n   a n d   f i v e 
+ 
+ 	 	 	 / /   s t i l l   i n   t h e   p o o l 
+ 
+ 	 	 	 f o r   ( i n t   i   =   1 ;   i   <   5 ;   i + + ) 
  
  	 	 	 { 
  
- 	 	 	 	 t r y 
+ 	 	 	 	 c o n n e c t i o n s [ i ]   =   n e w   M y S q l C o n n e c t i o n ( c o n n S t r ) ; 
  
- 	 	 	 	 { 
- 
- 	 	 	 	 	 c . O p e n ( ) ; 
- 
- 	 	 	 	 } 
- 
- 	 	 	 	 c a t c h   ( E x c e p t i o n   e x ) 
- 
- 	 	 	 	 { 
- 
- 	 	 	 	 } 
+ 	 	 	 	 c o n n e c t i o n s [ i ] . O p e n ( ) ; 
  
  	 	 	 } 
  
- 	 	 	 c . O p e n ( ) ; 
  
- 	 	 	 c . C l o s e ( ) ; * / 
+ 
+                         A s s e r t . A r e E q u a l ( 5 ,   i n U s e L i s t . C o u n t ) ; 
+ 
+                         A s s e r t . A r e E q u a l ( 5 ,   i d l e L i s t . C o u n t ) ; 
+ 
+ 
+ 
+                         A s s e r t . A r e E q u a l ( 0 ,   c l e a r i n g P o o l s . C o u n t ) ; 
+ 
+ 	 	 	 / /   n o w   t e l l   t h i s   c o n n e c t i o n   t o   c l e a r   i t s   p o o l 
+ 
+ 	 	 	 M y S q l C o n n e c t i o n . C l e a r P o o l ( c o n n e c t i o n s [ 0 ] ) ; 
+ 
+                         A s s e r t . A r e E q u a l ( 1 ,   c l e a r i n g P o o l s . C o u n t ) ; 
+ 
+ 	 	 	 A s s e r t . A r e E q u a l ( 0 ,   i d l e L i s t . C o u n t ) ; 
+ 
+ 
+ 
+ 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   5 ;   i + + ) 
+ 
+ 	 	 	 	 c o n n e c t i o n s [ i ] . C l o s e ( ) ; 
+ 
+                         A s s e r t . A r e E q u a l ( 0 ,   c l e a r i n g P o o l s . C o u n t ) ; 
+ 
+ 	 	 } 
+ 
+ 
+ 
+ 	 	 [ T e s t ] 
+ 
+ 	 	 p u b l i c   v o i d   T e s t B a d C o n n e c t i o n s ( ) 
+ 
+ 	 	 { 
+ 
+ 	 	 	 M y S q l C o n n e c t i o n S t r i n g B u i l d e r   b u i l d e r   =   n e w 
+ 
+ 	 	 	 M y S q l C o n n e c t i o n S t r i n g B u i l d e r ( ) ; 
+ 
+ 
+ 
+ 	 	 	 b u i l d e r . P o o l i n g   =   t r u e ; 
+ 
+ 	 	 	 b u i l d e r . S e r v e r   =   " x x x x x x x x " ;   / /   o n e   t h a t   d e f i n i t e l y   d o e s   n o t   e x i s t . 
+ 
+ 	 	 	 b u i l d e r . U s e r I D   =   " w h o e v e r " ; 
+ 
+ 	 	 	 b u i l d e r . P a s s w o r d   =   " w h a t e v e r " ; 
+ 
+ 
+ 
+ 	 	 	 i n t   n u m b e r O f C o n n e c t i o n s   =   1 ; 
+ 
+ 
+ 
+ 	 	 	 f o r   ( i n t   i   =   0 ;   i   <   n u m b e r O f C o n n e c t i o n s ;   + + i ) 
+ 
+ 	 	 	 { 
+ 
+ 	 	 	 	 u s i n g   ( M y S q l C o n n e c t i o n   c o n n e c t i o n   =   n e w   M y S q l C o n n e c t i o n ( b u i l d e r . C o n n e c t i o n S t r i n g ) ) 
+ 
+ 	 	 	 	 { 
+ 
+ 	 	 	 	 	 t r y 
+ 
+ 	 	 	 	 	 { 
+ 
+ 	 	 	 	 	 	 c o n n e c t i o n . O p e n ( ) ; 
+ 
+ 	 	 	 	 	 	 A s s e r t . F a i l ( " C o n n e c t i o n   s h o u l d   t h r o w   a n   e x c e p t i o n . " ) ; 
+ 
+ 	 	 	 	 	 } 
+ 
+ 	 	 	 	 	 c a t c h   ( E x c e p t i o n   e x ) 
+ 
+ 	 	 	 	 	 { 
+ 
+ 	 	 	 	 	 } 
+ 
+ 	 	 	 	 } 
+ 
+ 	 	 	 	 T h r e a d . S l e e p ( 5 0 ) ; 
+ 
+ 	 	 	 } 
+ 
+ 	 	 	 M y S q l C o n n e c t i o n . C l e a r A l l P o o l s ( ) ; 
  
  	 	 } 
  
